@@ -840,6 +840,7 @@ SEXP GPB_CreateREModel_R(SEXP ndata,
 	SEXP matrix_inversion_method,
 	SEXP seed,
 	SEXP num_parallel_threads,
+	SEXP GPU_use,
 	SEXP has_weights,
 	SEXP weights,
 	SEXP likelihood_learning_rate) {
@@ -893,6 +894,7 @@ SEXP GPB_CreateREModel_R(SEXP ndata,
 		matrix_inversion_method_ptr,
 		Rf_asInteger(seed),
 		Rf_asInteger(num_parallel_threads),
+		Rf_asLogical(GPU_use),
 		Rf_asLogical(has_weights),
 		R_REAL_PTR(weights),
 		Rf_asReal(likelihood_learning_rate),
@@ -1383,7 +1385,7 @@ static const R_CallMethodDef CallEntries[] = {
   {"LGBM_BoosterSaveModel_R"          , (DL_FUNC)&LGBM_BoosterSaveModel_R          , 4},
   {"LGBM_BoosterSaveModelToString_R"  , (DL_FUNC)&LGBM_BoosterSaveModelToString_R  , 4},
   {"LGBM_BoosterDumpModel_R"          , (DL_FUNC)&LGBM_BoosterDumpModel_R          , 3},
-  {"GPB_CreateREModel_R"              , (DL_FUNC)&GPB_CreateREModel_R              , 31},
+  {"GPB_CreateREModel_R"              , (DL_FUNC)&GPB_CreateREModel_R              , 32},
   {"GPB_REModelFree_R"                , (DL_FUNC)&GPB_REModelFree_R                , 1},
   {"GPB_SetOptimConfig_R"             , (DL_FUNC)&GPB_SetOptimConfig_R             , 30},
   {"GPB_CanCalculateStandardErrorsCovPars_R", (DL_FUNC)&GPB_CanCalculateStandardErrorsCovPars_R, 2},
