@@ -648,6 +648,7 @@ GPBOOST_C_EXPORT SEXP LGBM_BoosterDumpModel_R(
 * \param matrix_inversion_method Method which is used for matrix inversion
 * \param seed Seed used for model creation (e.g., random ordering in Vecchia approximation)
 * \param num_parallel_threads Number of parallel threads for OMP
+* \param GPU_use If TRUE, GPU acceleration will be used if supported.
 * \param has_weights True, if sample weights should be used
 * \param weights Sample weights
 * \param likelihood_learning_rate Likelihood learning rate for generalized Bayesian inference (only non-Gaussian likelihoods)
@@ -682,6 +683,7 @@ GPBOOST_C_EXPORT SEXP GPB_CreateREModel_R(
 	SEXP matrix_inversion_method,
 	SEXP seed,
 	SEXP num_parallel_threads,
+	SEXP GPU_use,
 	SEXP has_weights,
 	SEXP weights,
 	SEXP likelihood_learning_rate
