@@ -1348,6 +1348,7 @@ GPBOOST_C_EXPORT int LGBM_NetworkInitWithFunctions(int num_machines,
 * \param matrix_inversion_method Method which is used for matrix inversion
 * \param seed Seed used for model creation (e.g., random ordering in Vecchia approximation)
 * \param num_parallel_threads Number of parallel threads for OMP
+* \param GPU_use If TRUE, GPU acceleration will be used if supported.
 * \param has_weights True, if sample weights should be used
 * \param weights Sample weights
 * \param likelihood_learning_rate Likelihood learning rate for generalized Bayesian inference (only non-Gaussian likelihoods)
@@ -1382,6 +1383,7 @@ GPBOOST_C_EXPORT int GPB_CreateREModel(int32_t num_data,
     const char* matrix_inversion_method,
     int seed,
     int num_parallel_threads,
+    bool GPU_use,
     bool has_weights,
     const double* weights,
     double likelihood_learning_rate,
