@@ -46,6 +46,7 @@ namespace GPBoost {
 		const char* matrix_inversion_method,
 		int seed,
 		int num_parallel_threads,
+		bool GPU_use,
 		bool has_weights,
 		const double* weights,
 		double likelihood_learning_rate) {
@@ -81,7 +82,7 @@ namespace GPBoost {
 				ind_effect_group_rand_coef, num_re_group_rand_coef, drop_intercept_group_rand_effect,
 				num_gp, gp_coords_data, dim_gp_coords, gp_rand_coef_data, num_gp_rand_coef, cov_fct, cov_fct_shape, gp_approx,
 				cov_fct_taper_range, cov_fct_taper_shape, num_neighbors, vecchia_ordering, num_ind_points, cover_tree_radius, ind_points_selection,
-				likelihood, likelihood_additional_param, matrix_inversion_method, seed, num_parallel_threads,
+				likelihood, likelihood_additional_param, matrix_inversion_method, seed, num_parallel_threads, GPU_use,
 				has_weights, weights, likelihood_learning_rate));
 			num_cov_pars_ = re_model_sp_->num_cov_par_;
 			num_sets_fixed_effects_ = re_model_sp_->num_sets_fixed_effects_;
@@ -92,7 +93,7 @@ namespace GPBoost {
 				ind_effect_group_rand_coef, num_re_group_rand_coef, drop_intercept_group_rand_effect,
 				num_gp, gp_coords_data, dim_gp_coords, gp_rand_coef_data, num_gp_rand_coef, cov_fct, cov_fct_shape, gp_approx,
 				cov_fct_taper_range, cov_fct_taper_shape, num_neighbors, vecchia_ordering, num_ind_points, cover_tree_radius, ind_points_selection,
-				likelihood, likelihood_additional_param, matrix_inversion_method, seed, num_parallel_threads,
+				likelihood, likelihood_additional_param, matrix_inversion_method, seed, num_parallel_threads, GPU_use,
 				has_weights, weights, likelihood_learning_rate));
 			num_cov_pars_ = re_model_sp_rm_->num_cov_par_;
 			num_sets_fixed_effects_ = re_model_sp_rm_->num_sets_fixed_effects_;
@@ -103,7 +104,7 @@ namespace GPBoost {
 				ind_effect_group_rand_coef, num_re_group_rand_coef, drop_intercept_group_rand_effect,
 				num_gp, gp_coords_data, dim_gp_coords, gp_rand_coef_data, num_gp_rand_coef, cov_fct, cov_fct_shape, gp_approx,
 				cov_fct_taper_range, cov_fct_taper_shape, num_neighbors, vecchia_ordering, num_ind_points, cover_tree_radius, ind_points_selection,
-				likelihood, likelihood_additional_param, matrix_inversion_method, seed, num_parallel_threads,
+				likelihood, likelihood_additional_param, matrix_inversion_method, seed, num_parallel_threads, GPU_use,
 				has_weights, weights, likelihood_learning_rate));
 			num_cov_pars_ = re_model_den_->num_cov_par_;
 			num_sets_fixed_effects_ = re_model_den_->num_sets_fixed_effects_;
